@@ -3,6 +3,7 @@
 # vim: set expandtab ts=4 sw=4:
 
 import numpy as np
+from copy import deepcopy
 from . import util, design, regressors
 from anamnesis import AbstractAnam, register_class
 
@@ -77,6 +78,97 @@ class AbstractGLMData(AbstractAnam):
         new_shape = tuple((X.shape[0], *self.orig_shape[1:]))
         return X.reshape(*new_shape)
 
+    def drop(self, inds, inplace=False):
+
+        if inplace is False:
+            ret = deepcopy(self)
+        else:
+            # Just a pointer
+            ret = self
+
+        for key in ret.info.keys():
+            if isinstance(ret.info[key], (list, np.ndarray)) and len(ret.info[key]) == ret.data.shape[0]:
+                ret.info[key] = np.delete(np.array(ret.info[key]), inds)
+
+        ret.data = np.delete(ret.data, inds, axis=0)
+        ret.info['num_observations'] = ret.data.shape[0]
+
+        return ret
+
+
+class GLMData:
+
+    def __init__(self, **kwargs):
+
+        # Store data if passed in
+        if 'data' in kwargs:
+            self.data = kwargs['data']
+            # Make sure data is at least 2d
+            if np.ndim(self.data) == 1:
+                self.data = self.data[:, None]
+            del kwargs['data']
+        else:
+            self.data = None
+
+        # Arbitrary information stored in meta
+        self.meta = {}
+        if 'dim_labels' in kwargs.keys():
+            # Set default dimension labels if not defined
+            self.meta['dim_labels'] = kwargs['dim_labels']
+        elif self.data is not None and 'dim_labels' not in self.info:
+            self.meta['dim_labels'] = list(('Observations', *['dim'+chr(65+ii) for ii in range(self.data.ndim-1)]))
+
+        if 'meta' in kwargs:
+            for key, item in kwargs['meta'].items():
+                self.meta[key] = item
+        self.meta['num_observations'] = self.data.shape[0]
+
+        # Covariates MUST be list of same length of data
+        self.covariates = {}
+        for key in kwargs.keys():
+            if key not in ('data', 'meta', 'dim_labels'):
+                if len(kwargs[key]) == self.data.shape[0]:
+                    self.covariates[key] = list(kwargs[key])
+
+    @property
+    def orig_shape(self):
+        "The shape of the original data"
+        return self.data.shape
+
+    @property
+    def squash_shape(self):
+        "The shape of the data after reshaping before model fitting."
+        return (self.data.shape[0], np.prod(self.data.shape[1:]))
+
+    @property
+    def num_observations(self):
+        "The number of observations in the data, this is the first dimension of the input data"
+        return self.data.shape[0]
+
+    def get_2d_data(self):
+        return self.data.reshape(self.squash_shape)
+
+    def unsquash_array(self, X):
+        new_shape = tuple((X.shape[0], *self.orig_shape[1:]))
+        return X.reshape(*new_shape)
+
+    def drop(self, inds, inplace=False):
+
+        if inplace is False:
+            ret = deepcopy(self)
+        else:
+            # Just a pointer
+            ret = self
+
+        for key in ret.info.keys():
+            if isinstance(ret.info[key], (list, np.ndarray)) and len(ret.info[key]) == ret.data.shape[0]:
+                ret.info[key] = np.delete(np.array(ret.info[key]), inds)
+
+        ret.data = np.delete(ret.data, inds, axis=0)
+        ret.info['num_observations'] = ret.data.shape[0]
+
+        return ret
+
 
 class TrialGLMData(AbstractGLMData):
     """
