@@ -1,5 +1,6 @@
 #!/usr/bin/python
 
+import yaml
 import warnings
 import numpy as np
 from . import viz, regressors, util, fit, data
@@ -153,17 +154,27 @@ class GLMDesign(AbstractAnam):
         "The model degrees of freedom (num observations - num regressors)"
         return self.num_observations - self.design_matrix_rank
 
+    @property
+    def leverage(self):
+        return np.diag(self.design_matrix.dot(np.linalg.pinv(self.design_matrix)))
+
     # Visualisations
     def print_summary(self):
         "Print a text summary of the model."
 
         viz.print_contrast_table(self.contrasts,
                                  contrast_names=self.contrast_names,
-                                 regressor_name=self.regressor_names,
-                                 f_tests=self.f_tests,
+                                 regressor_names=self.regressor_names,
+                                 f_tests=self.ftest_list,
                                  ftest_names=self.ftest_names)
 
-    def plot_efficiency(self, showfig=False, savepath=None):
+    def summarise_design(self):
+        print('\n')
+        print(viz.summarise_regressor_list(self.regressor_list))
+        print('\n')
+        print(viz.summarise_contrasts(self.contrasts, self.contrast_names, self.regressor_names))
+
+    def plot_efficiency(self, show=True, savepath=None):
         "Plot the correlation matrix and singular values from the design matrix"
 
         fig = viz.plot_design_efficiency(self.design_matrix,
@@ -174,12 +185,15 @@ class GLMDesign(AbstractAnam):
         if savepath is not None:
             plt.savefig(savepath, dpi=300)
 
-        if showfig:
+        if show:
             plt.show()
+        else:
+            plt.close(fig)
 
         return fig
 
-    def plot_summary(self, show=True, savepath=None, summary_lines=True):
+    def plot_summary(self, show=True, savepath=None, summary_lines=True,
+                     fig=None, ax=None, figargs=dict()):
         "Plot a summary of the design matrix and contrasts"
 
         fig = viz.plot_design_summary(self.design_matrix,
@@ -188,7 +202,24 @@ class GLMDesign(AbstractAnam):
                                       contrast_names=self.contrast_names,
                                       ftests=self.ftests,
                                       ftest_names=self.ftest_names,
-                                      summary_lines=summary_lines)
+                                      summary_lines=summary_lines,
+                                      fig=fig, ax=ax, figargs=figargs)
+
+        import matplotlib.pyplot as plt
+        if savepath is not None:
+            plt.savefig(savepath, dpi=300)
+
+        if show:
+            plt.show()
+        else:
+            plt.close(fig)
+
+        return fig
+
+    def plot_leverage(self, thresh=5, show=True, savepath=None):
+        """Plot a summary of the leverage across observations in the design matrix"""
+
+        fig = viz.plot_leverage(self.leverage, thresh=thresh)
 
         import matplotlib.pyplot as plt
         if savepath is not None:
@@ -303,6 +334,7 @@ class GLMDesign(AbstractAnam):
             ret.regressor_names = np.arange(ret.design_matrix.shape[1]).astype(str)
         else:
             ret.regressor_names = regressor_names
+        ret.regressor_list = []
 
         # Set default contrast names if not provided
         if contrast_names is None:
@@ -459,8 +491,11 @@ class DesignConfig:
 
     def add_ftest(self, **kwargs):
         if 'name' not in kwargs:
-            kwargs['name'] = 'Regressor{0}'.format(self.regressors.__len__())
-        kwargs['values'] = list(kwargs['values'])
+            kwargs['name'] = 'Contrast{0}'.format(self.contrasts.__len__())
+        if isinstance(kwargs['values'], str):
+            kwargs['values'] = list(map(float, kwargs['values'].split(' ')))
+        elif isinstance(kwargs['values'], list):
+            kwargs['values'] = list(map(float, kwargs['values']))
         self.ftests.append(kwargs)
 
     def _check_regressor(self, reg):
@@ -495,9 +530,17 @@ class DesignConfig:
 
     def _check_ftest(self, ft):
         isgood = True  # Start positive
-        if len(ft['values']) != self.num_contrasts:
-            print('F-test: {0} - values mis-matched to num_contrasts'.format(ft['name']))
+        if isinstance(ft['values'], dict):
+            for key in ft['values']:
+                if key not in self.contrast_names:
+                    print("F-Test: {0} - contrast name ({1}) not found".format(ft['name'], key))
+        else:
+            if len(ft['values']) != self.num_regressors:
+                print('F-Test: {0} - values mis-matched to num_contrasts'.format(ft['name']))
             isgood = False
+        #if len(ft['values']) != self.num_contrasts:
+        #    print('F-test: {0} - values mis-matched to num_contrasts'.format(ft['name']))
+        #    isgood = False
         return isgood
 
     def add_simple_contrasts(self, reg_names=None):
@@ -508,8 +551,9 @@ class DesignConfig:
         for reg in reg_names:
             for ind, r in enumerate(self.regressors):
                 if r['name'] == reg:
-                    values = list(np.zeros((self.num_regressors,)))
-                    values[ind] = 1
+                    #values = list(np.zeros((self.num_regressors,)))
+                    #values[ind] = 1
+                    values = {r['name']: 1}
                     self.add_contrast(name=r['name'], values=values)
 
     def add_mean_effects(self, groups, basename='Mean{0}'):
@@ -529,10 +573,37 @@ class DesignConfig:
 
     def to_yaml(self):
         self.validate()
-        import yaml
         conf = self.to_dict()
         return yaml.dump(conf, default_flow_style=None, explicit_start=True, width=1000)
 
+    def to_yaml_file(self, fname):
+        """Save a copy of the internal store in a specified yaml file."""
+        self.validate()
+        conf = self.to_dict()
+        with open(fname, 'w') as f:
+            return yaml.dump(conf, f, default_flow_style=None, explicit_start=True, width=1000)
+
+    @classmethod
+    def from_yaml_file(cls, fname):
+        """Create and return a new DesignConfig object with options loaded from a yaml file."""
+        with open(fname, 'r') as f:
+            cfg = [d for d in yaml.load_all(f, Loader=yaml.FullLoader)]
+            if len(cfg) == 1:
+                ret = cls().from_dict(cfg[0])
+            else:
+                raise RuntimeError('Multiple configs found in file')
+
+        return ret
+
+    @classmethod
+    def from_dict(cls, indict):
+        ret = cls()
+        ret.name = list(indict.keys())[0]
+        ret.regressors = indict[ret.name]['regressors']
+        ret.contrasts = indict[ret.name]['contrasts']
+        ret.validate()
+        return ret
+
     def validate(self):
 
         r = np.alltrue([self._check_regressor(reg) for reg in self.regressors])
@@ -551,7 +622,7 @@ class DesignConfig:
         regressors = util.regressors_from_datainfo(self.regressors, info)
         contrasts = util.contrasts_from_dict(self.contrasts, regs=regressors)
         if self.ftests is not None:
-            ftests = util.contrasts_from_dict(self.ftests)
+            ftests = util.ftests_from_dict(self.ftests, regs=contrasts)
         else:
             ftests = None
 
