@@ -3,6 +3,7 @@
 # vim: set expandtab ts=4 sw=4:
 
 import numpy as np
+from tabulate import tabulate
 import matplotlib.pyplot as plt
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 from matplotlib import cm
@@ -22,7 +23,7 @@ def print_contrast_table(contrasts,
     if regressor_names is None:
         np.arange(num_regressors).astype(str)
 
-    if f_tests is None:
+    if len(f_tests) == 0 or f_tests is None:
         num_ftests = None
 
     elif ftest_names is None:
@@ -57,6 +58,34 @@ def print_contrast_table(contrasts,
             print(template.format(*tmp))
 
 
+def summarise_regressor_list(regressor_list, tablefmt='simple'):
+
+    tdata = [(r.name,
+              r.rtype,
+              len(r.values),
+              r.values.min(),
+              r.values.mean(),
+              r.values.max()) for r in regressor_list]
+    headers = ['Regressor', 'Type', 'NumValues', 'MinValue', 'MeanValue', 'MaxValue']
+    print(tabulate(tdata, headers=headers, tablefmt=tablefmt))
+
+
+def summarise_contrast_list(contrast_list, tablefmt='simple'):
+
+    tdata = [(c.name,
+              c.ctype,
+              c.values) for c in contrast_list]
+    print(tabulate(tdata, headers=['Contrast', 'Type', 'Values'], tablefmt=tablefmt))
+
+
+def summarise_contrasts(contrasts, contrast_names, regressor_names, tablefmt='simple'):
+
+    tdata = [(contrast_names[c], *contrasts[:, c]) for c in range(len(contrast_names))]
+    headers = ['', *regressor_names]
+    print('Contrasts in rows, Regressors in columns')
+    print(tabulate(tdata, headers=headers, tablefmt=tablefmt))
+
+
 def plot_design_efficiency(design_matrix, regressor_names=None,
                            normalise_singlar_values=True, figargs=dict()):
 
@@ -154,7 +183,7 @@ def plot_regressor_correlation(design_matrix, regressor_names=None, ax=None, fig
 
     num_regressors = design_matrix.shape[1]
 
-    cmap = cm.RdBu_r
+    cmap = cm.RdBu_r.copy()
     cmap.set_bad(color=[0.8, 0.8, 0.8])
 
     corr_mat = np.corrcoef(design_matrix.T)
@@ -184,7 +213,8 @@ def plot_regressor_correlation(design_matrix, regressor_names=None, ax=None, fig
 def plot_design_summary(design_matrix, regressor_names,
                         contrasts=None, contrast_names=None,
                         ftests=None, ftest_names=None,
-                        summary_lines=True, figargs=dict()):
+                        summary_lines=True, figargs=dict(),
+                        fig=None, ax=None):
 
     num_observations, num_regressors = design_matrix.shape
 
@@ -192,7 +222,18 @@ def plot_design_summary(design_matrix, regressor_names,
     if 'figsize' not in figargs:
         figargs['figsize'] = (12, 6)
 
-    f = plt.figure(**figargs)
+    if (fig is None) and (ax is None):
+        fig = plt.figure(**figargs)
+
+    elif (fig is None) and (ax is not None):
+        fig = ax.figure
+
+    if ax is None:
+        # Plot design matrix
+        if contrasts is None:
+            ax = plt.axes([.1, .2, .8, .65])
+        else:
+            ax = plt.axes([.1, .05, .8, .85])
 
     if contrasts is not None:
         # Add rows for contrast matrix, one contrast takes about 5% of the design matrix
@@ -224,12 +265,6 @@ def plot_design_summary(design_matrix, regressor_names,
 
     vm = np.max((design_matrix.min(), design_matrix.max()))
 
-    # Plot design matrix
-    if contrasts is None:
-        ax = plt.axes([.1, .2, .8, .65])
-    else:
-        ax = plt.axes([.1, .05, .8, .85])
-    #cax = ax.pcolor( pad_design,cmap=cm.binary )
     cax = ax.pcolor(pad_design, cmap=cm.coolwarm,
                     vmin=-vm, vmax=vm)
     ax.set_xlabel('Regressors')
@@ -263,7 +298,7 @@ def plot_design_summary(design_matrix, regressor_names,
     cb_step_x = new_fcols/pad_design.shape[1]
     cb_step_x = .92 - cb_step_x + cb_step_x/3
 
-    cb_ax = f.add_axes([cb_step_x, cb_step_y, .025, .9-cb_step_y])
+    cb_ax = fig.add_axes([cb_step_x, cb_step_y, .025, .9-cb_step_y])
     plt.colorbar(cax, ax=ax, cax=cb_ax)
 
     # Turn box off
@@ -303,8 +338,8 @@ def plot_design_summary(design_matrix, regressor_names,
         for ii in range(len(regressor_names)):
             for jj in range(len(contrast_names)):
                 if ii == 0:
-                    ax.text(.5, new_rows-((2+jj) * step), 'C: ' + contrast_names[jj],
-                            horizontalalignment='center')
+                    ax.text(0, new_rows-((2+jj) * step), 'C{0}: '.format(jj+1) + contrast_names[jj],
+                            horizontalalignment='left')
 
                 ax.text(1.5+ii, new_rows-((2+jj)*step), str(contrasts[jj, ii]),
                         horizontalalignment='center')
@@ -316,4 +351,33 @@ def plot_design_summary(design_matrix, regressor_names,
             for jj in range(len(contrast_names)):
                 ax.text(num_regressors+2.5+ii, new_rows-((2+jj)*step), str(ftests[ii, jj]))
 
-    return f
+    return fig
+
+
+def plot_leverage(leverage, thresh=5):
+
+    fig = plt.figure(figsize=(12, 5))
+    plt.plot(leverage, '.', label='leverage')
+    plt.plot((0, len(leverage)),
+             (np.median(leverage), np.median(leverage)),
+             '--', label='median leverage')
+    plt.plot((0, len(leverage)),
+             (thresh*np.median(leverage), thresh*np.median(leverage)),
+             '--', label='{0}*median leverage'.format(thresh))
+    plt.ylabel('Leverage')
+    plt.xlabel('Observations')
+    plt.ylim(0)
+    plt.xlim(0, len(leverage))
+    for tag in ['top', 'right']:
+        plt.gca().spines[tag].set_visible(False)
+    plt.legend(frameon=False)
+
+    outliers = np.where(leverage > thresh*np.median(leverage))[0]
+    for ii in outliers:
+        plt.text(ii, leverage[ii], '  ' + str(ii), ha='left', va='center')
+
+    msg = 'Leverage per observation\nvalues with high leverage may '
+    msg += 'excessively influence the outcome of the regression'
+    plt.title(msg, pad=10)
+
+    return fig
